#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: wifilist.py 2602 2010-06-02 19:02:44Z v13 $

__version__ = "$Id: wifilist.py 2602 2010-06-02 19:02:44Z v13 $"

from PyQt4 import QtCore
from PyQt4.QtCore import Qt
from PyQt4.QtGui import *
from PyQt4.QtCore import *

import math
import sys

import data
import channels

import maemo5

import sip

class Line(QWidget):
    def __init__(self, *args, **kwargs):
	QWidget.__init__(self, *args, **kwargs)

	self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)

	self.setMinimumHeight(3)
	self.setMaximumHeight(3)

    def paintEvent(self, event):
	painter=QPainter(self)

	painter.setRenderHints(painter.Antialiasing | \
	    painter.TextAntialiasing | \
	    painter.HighQualityAntialiasing
	    )

#	col=Qt.gray
#	pen=QPen(col)
#	pen.setWidth(2)
#	painter.setPen(pen)

	self.setPen(painter)

	painter.drawLine(0,1,self.width(),1)

    def sizeHint(self):
	ret=QSize(20, 3)
	return(ret)

class LineDash(Line):
    def setPen(self, painter):
	pen=painter.pen()
	pen.setCapStyle(Qt.RoundCap)
	pen.setCosmetic(True)
#	pen.setStyle(Qt.DotLine)
	pen.setStyle(Qt.DashLine)
	painter.setPen(pen)

	return(pen)

class LineLine(Line):
    def setPen(self, painter):
	pen=painter.pen()
	pen.setCapStyle(Qt.RoundCap)
	pen.setCosmetic(True)
	pen.setStyle(Qt.SolidLine)
	painter.setPen(pen)

	return(pen)

class WifiListData:
    def __init__(self):
	self.olddata=None
	self.data={'items':[], 'data':{}}
	self._sort_dt=None  # Used for the custom-sorting

#	self.update_data()

    # Custom sorting function
    #
    # Rationale:
    #	Display first the good-quality signals
    #	Try not to change the location of each ssid by using 5-steps of
    #	sorting (quality/20) and use essid-name sorting inside each step
    #	This way, low-strength signals will not annoy the user
    def sort_cmp(self, a, b):
	dt=self._sort_dt
	dt1=int(dt[a]['quality'] / 20)
	dt2=int(dt[b]['quality'] / 20)

	if dt1!=dt2:
	    ret=dt2-dt1
	else:
	    s1=dt[a]['essid']
	    s2=dt[b]['essid']
	    if s1<s2:
		ret=-1
	    elif s1>s2:
		ret=1
	    else:
		ret=0

#	print "srt:", ret
	return(ret)

    def sort_cmp_channel(self, a, b):
	dt=self._sort_dt
	dt1=dt[a]['freq']
	dt2=dt[b]['freq']

	if dt1!=dt2:
	    ret=dt1-dt2
	else:
	    if a<b:
		ret=-1
	    elif a>b:
		ret=1
	    else:
		ret=0
	
	return(ret)

    def updateData(self):
	#print "update_data"
	self.olddata=self.data
	self.data={'items':[], 'data':{}}

	dt0=data.getone()
	dt={}
	for i in dt0:
	    d=dt0[i]
	    if not dt.has_key(d['essid']):
		dt[d['essid']]={
		    'essid':	d['essid'],
		    'aps':	{},
		    'quality':	0,
		    }

	    auth=[]
	    enc=[]
	    for e in d['enc']:
		for e2 in e['auth']:
		    try:
			auth.index(e2)
		    except:
			auth.append(e2)
		enc.append(e['enc'])

	    dt[d['essid']]['aps'][d['bssid']]={
		'bssid':    d['bssid'],
		'freq':	    int(d['freq']['freq']),
		'quality':  d['quality']['quality'],
		'auth':	    auth,
		'enc':	    enc,
		'channel':  channels.freqToChan(d['freq']['freq']),
		}

	    if dt[d['essid']]['quality'] < d['quality']['quality']:
		dt[d['essid']]['quality']=d['quality']['quality']

	for i in dt:
	    maxq=0
	    for k in dt[i]['aps']:
		q=dt[i]['aps'][k]['quality']
		if q>maxq:
		    maxq=q

	    dt[i]['quality']=maxq

	k=dt.keys()
	self._sort_dt=dt
	k.sort(cmp=self.sort_cmp)

	self.data['items']=k
	self.data['data']=dt

	for i in dt:
	    k=dt[i]['aps'].keys()
	    self._sort_dt=dt[i]['aps']
	    k.sort(cmp=self.sort_cmp_channel)
	    dt[i]['items']=k
	    
    def getData(self):
	ret=[]

	for i in self.data['items']:
	    ret.append(self.data['data'][i])

	return(ret)
	

class WifiListQuality(QWidget):
    """
    A Widget to display the quality of a link
    """
    def __init__(self, *args, **kwargs):
	QWidget.__init__(self, *args, **kwargs)

	self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
#	    QSizePolicy.Minimum)

	self.setMinimumHeight(32)
	self.setMinimumWidth(150)

	self.steps=10

	self._value=2

    def paintEvent(self, event):
	painter=QPainter(self)

	painter.setRenderHints(painter.Antialiasing | \
	    painter.TextAntialiasing | \
	    painter.HighQualityAntialiasing
	    )

	font0=QFont(painter.font())

	font_perc=QFont(font0)
	#font_perc.setPointSizeF(14)
	font_perc.setPixelSize(self.height() * 2 / 3)
	font_perc_m=QFontMetrics(font_perc)

	col_perc=Qt.white
	pen_perc=QPen(col_perc)

	x0=0
	y0=0

	# Margin between graph and text
	txt_margin=4

	# Space required for text plus some margin
	txt_space = font_perc_m.width("IXXXX") + txt_margin

	# The width of the bar plus its borders
	bar_width=self.width() - txt_space

	# Space between each "cell"
	spacing=2
	# Border-line width
	line_width=1
	# Space between cells and border
	cell_padding_x=3
	cell_padding_y=3
	# Step between each cell: 
	x_step = math.floor(
	    (bar_width - (2*cell_padding_x) - ((self.steps-1) * spacing) - \
		(line_width * 2)) / self.steps) + spacing

	# Since we did some calculations and we need integer results,
	# recalculate the width of the bar by multuplying the cell size
	# with the number of cells
	width = (x_step * self.steps) + (line_width*2) + (spacing*2)

	# Draw the text
	painter.setPen(pen_perc)
	painter.setFont(font_perc)
	txt_rect = QRect(self.width() - txt_space, 2, txt_space,
	    font_perc_m.height())
	painter.drawText(txt_rect, Qt.AlignRight, "%s%%" % (self._value))

	# Draw the graph
	w=x_step - spacing
	h=self.height() - (cell_padding_y*2) - (line_width*2)

	for step in xrange(self.steps):
	    x=x0 + (step*x_step) + line_width + cell_padding_x
	    y=y0 + cell_padding_y + line_width

	    rect=QRect(x, y, w, h)

	    # Fill color
	    colfill=QColor(
		0x00,
		0x44 + 0xbb * ((1.0*step / self.steps)**2),
		0x00)

	    # Fill color for "empty" cells
	    colfill0=QColor(0x00, 0x22, 0x00)

	    # Brushes for cells: One for the filled cell, another for the
	    # non-filled cell
	    br=QBrush(QColor(colfill))
	    br0=QBrush(QColor(colfill0))

	    t1=step*100/self.steps
	    # Leave this "<". "<=" doesn't look beautifull with
	    # (e.g) ten steps and 90%
	    if t1 < self._value:
		painter.fillRect(rect, br)
	    else:
		painter.fillRect(rect, br0)

	# Draw the outline of cells
	# Outline color
	colol=QColor(0x33, 0x99, 0x33)
	painter.setPen(QPen(colol))

	rect=QRect(x0,y0,width, self.height())
	painter.drawRoundedRect(rect, 4, 5)

    def setValue(self, value):
	self._value=value

    def sizeHint(self):
	ret=QSize(180, 32)
	return(ret)

class WifiListWidgetAP(QWidget):
    def __init__(self, *args, **kwargs):
	QWidget.__init__(self, *args, **kwargs)

	self.setObjectName("ap")

	self.create()

    def create_widgets(self):
	# Calculate the font size (80% of the original)
	self.lBssid=QLabel(self)
	self.lBssid.setObjectName("bssid")
	font=QFont(self.lBssid.font())
	font.setPointSizeF(font.pointSizeF()*0.8)
	self.lBssid.setFont(font)

	self.lChannel=QLabel(self)
	self.lChannel.setObjectName("channel")
	self.lChannel.setFont(font)

	self.lAuth=QLabel(self)
	self.lAuth.setObjectName('auth')
	font=QFont(self.lAuth.font())
	font.setPointSizeF(font.pointSizeF()*0.8)
	self.lAuth.setFont(font)

	self.pQuality=WifiListQuality(self)

    def arrange_portrait(self):
	self.l0=QVBoxLayout(self)

	l1=QHBoxLayout()
	self.l1=l1
	self.l0.addLayout(l1)

	l2=QHBoxLayout()
	self.l2=l2
	self.l0.addLayout(l2)

	l3=QHBoxLayout()
	self.l3=l3
	self.l0.addLayout(l3)

	self.l1.addSpacing(20)
	l1.addWidget(self.lBssid)

	self.l1.addSpacing(20)
	l1.addWidget(self.lChannel)

	self.l2.addSpacing(20)
	l2.addWidget(self.lAuth)
	self.l2.addSpacing(20)
	l2.addWidget(self.pQuality)


	self.l3.addSpacing(10)
	line=LineDash(self)
	line.setObjectName('line2')
	self.l3.addWidget(line)

#	spacer=QSpacerItem(1, 1, QSizePolicy.Expanding)
#	self.l0.addItem(spacer)

#	l2=QVBoxLayout()
#	self.l2=l2
#	self.l0.addLayout(l2)
#
#	spacer=QSpacerItem(1, 1, vPolicy=QSizePolicy.Expanding)
#	self.l2.addItem(spacer)

#	spacer=QSpacerItem(1, 1, QSizePolicy.Expanding)
#	self.l2.addItem(spacer)

    def arrange_portrait_old(self):
#	print "portrait"
	self.l0=QHBoxLayout(self)

	l1=QVBoxLayout()
	self.l1=l1
	self.l0.addLayout(l1)

#	self.l0.addSpacing(20)

	l1.addWidget(self.lBssid)

#	self.l0.addSpacing(20)

	l1.addWidget(self.lChannel)

#	self.l0.addSpacing(20)

	l1.addWidget(self.lAuth)

	spacer=QSpacerItem(1, 1, QSizePolicy.Expanding)
	self.l0.addItem(spacer)

	l2=QVBoxLayout()
	self.l2=l2
	self.l0.addLayout(l2)

	spacer=QSpacerItem(1, 1, vPolicy=QSizePolicy.Expanding)
	self.l2.addItem(spacer)

	self.l2.addWidget(self.pQuality)

	spacer=QSpacerItem(1, 1, vPolicy=QSizePolicy.Expanding)
	self.l2.addItem(spacer)

    def arrange_landscape(self):
#	print "landscape"
	self.l0=QHBoxLayout(self)

	self.l0.addSpacing(20)

	self.l0.addWidget(self.lBssid)

	self.l0.addSpacing(20)

	self.l0.addWidget(self.lChannel)

	self.l0.addSpacing(20)

	self.l0.addWidget(self.lAuth)

	spacer=QSpacerItem(1, 1, QSizePolicy.Expanding)
	self.l0.addItem(spacer)

	self.l0.addWidget(self.pQuality)

    def create(self):
	self.setAutoFillBackground(True)
#	pal=self.palette()

	self.create_widgets()

	if maemo5.isPortrait():
	    self.arrange_portrait()
	else:
	    self.arrange_landscape()

	self.l0.setContentsMargins(0,0,0,0)

#	self.updateGeometry()
#	print "done", self.sizeHint()

    def enc_str(self, st):
	""" Tranform the output of scan2.py
	(e.g. "WPA v1" -> "WPA")"""

	encshow={'WPA v1': 'WPA', 'WPA2 v1': 'WPA2'}

	if encshow.has_key(st):
	    ret=encshow[st]
	else:
	    ret=st
	
	return(ret)

    def setData(self, dt):
	"""
	dt:
	- bssid
	- freq
	- quality
	- auth
	- enc
	"""
	self.lBssid.setText(dt['bssid'])
	self.lChannel.setText("Ch %d - %s MHz" % (dt['channel'],  dt['freq']))
	self.pQuality.setValue(dt['quality'])

	auth=[]
	auth0=['Open', 'WEP', 'PSK']
	# First the well known in their order
	for a in auth0:
	    try:
		dt['auth'].index(a)
		auth.append(a)
	    except:
		pass
	# Then everything else
	for a in dt['auth']:
	    try:
		auth.index(a)
	    except:
		auth.append(a)

	enc=[]
	enc0=['None', 'WEP', 'WPA v1', 'WPA v2']
	# First the well known in their order
	for a in enc0:
	    try:
		dt['enc'].index(a)
		enc.append(a)
	    except:
		pass
	# Then everything else
	for a in dt['enc']:
	    try:
		enc.index(a)
	    except:
		enc.append(a)

	# Prety-print
	enc2=[]
	for e in enc:
	    enc2.append(self.enc_str(e))

	if len(enc2)==2 and \
	    ((enc2[0]=='WPA' and enc2[1]=='WPA2') or \
	    (enc2[0]=='WPA2' and enc2[1]=='WPA')):
	    enc2=['WPA1&2']

	if len(auth)==1 and auth[0]=='Open':
	    st="Open"
	elif len(auth)==1 and auth[0]=='WEP' and \
	    len(enc)==1 and enc2[0]=='WEP':
	    st="WEP"
#	elif len(auth)==1 and len(enc2)==1:
#	    st="%s (%s)" % (enc2[0], auth[0])
	else:
	    st=','.join(enc2) + ' (' + ','.join(auth) + ')'

	self.lAuth.setText(st)
#	self.lAuth.setText(', '.join(auth))
#	self.lEnc.setText(', '.join(enc2))

class WifiListWidget(QFrame):
    """
    Interesting items:

    - lEssid:	    The ESSID label
    - pQuality:	    The progress-bar for quality
    """
    def __init__(self, dt, *args, **kwargs):
	QFrame.__init__(self, *args, **kwargs)
	
	# A dictionary to hold the widgets for each AP
	self.apw={}

	self.w=None
	self.old_dt={}

	self.create(dt)

    def create(self, data):
	"""
	initial creation
	data:
	- essid
	- quality
	- aps:
	  - whatever WifiListWidgetAP.setData() needs
	"""

#	QVBoxLayout(self)

	self.l0=None

	self.lEssid=QLabel(self)
	self.pQuality=WifiListQuality(self)

	self.line1=LineLine(self)
	self.line1.setObjectName('line1')

#	QWidget { margin-top: 10px; border: 1px solid white;}
	self.setStyleSheet("""
	QLabel {
	    color: rgb(255, 192, 64);
	    }
	#ap QLabel {
	    color: rgb(128,192,255);
	    }
	#line1 {
	    color: rgb(255, 192, 64);
	    }
	#line2 {
	    color: rgb(64,92,128);
	    }
	""")

	self.l0=QVBoxLayout(self)
	self.l0.setContentsMargins(0,0,0,0)
	self.l1=QHBoxLayout()
	self.l1.setContentsMargins(0,0,0,0)

	self.l1.addWidget(self.lEssid)

	spacer=QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Minimum)
	self.l1.addItem(spacer)

	self.l1.addWidget(self.pQuality)

	if not maemo5.isPortrait():
	    self.l1.addSpacing(20)

	self.l0.addLayout(self.l1)

	self.l0.addWidget(self.line1)

	self.l2=QVBoxLayout()
	self.l2.setContentsMargins(0,0,0,0)

	self.l0.addLayout(self.l2)

	self.lEssid.setText(data['essid'])
	self.pQuality.setValue(data['quality'])

	for i in data['items']:
	    dt=data['aps'][i]
	    wlw=WifiListWidgetAP(self)
	    wlw.setData(dt)
	    self.apw[i]=wlw
	    self.l2.addWidget(wlw)

	spacer=QSpacerItem(1, 20, QSizePolicy.Minimum, QSizePolicy.Minimum)
	self.l0.addItem(spacer)

class WifiListMainWidget(QWidget):
    def __init__(self, parent, data, *args, **kwargs):
	QWidget.__init__(self, parent, *args, **kwargs)

	self.data=data

	self._create()

    def _create(self):
	self.l0=QVBoxLayout(self)
	self.l0.setContentsMargins(10, 10, 10, 10)

	self.wlws=[]
	for dt in self.data:
	    wlw=WifiListWidget(dt)
	    self.l0.addWidget(wlw)
	    self.wlws.append(wlw)

	spacer=QSpacerItem(1, 1, QSizePolicy.Preferred, QSizePolicy.Expanding)
	self.l0.addItem(spacer)

class WifiList(QScrollArea):
    def __init__(self, parent, *args, **kwargs):
	QScrollArea.__init__(self, parent, *args, **kwargs)

	self.setWidgetResizable(True)

	self.data=WifiListData()

	maemo5.connectOrientationChange(self.slotOrientationChanged)
	
    def update(self):
	self.data.updateData()

	self.update2()

    def update2(self):
	data=self.data.getData()

	w=WifiListMainWidget(self, data)

	self.setWidget(w)

    def slotOrientationChanged(self, dt):
	self.update2()

# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

