#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: win.py 2600 2010-06-02 18:53:23Z v13 $

__version__ = "$Id: win.py 2600 2010-06-02 18:53:23Z v13 $"

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import SIGNAL, Qt
from PyQt4.QtGui import *

import sys

import wifi, data, config
import maemo5

from win_common import MyMsgDialog
from win_time import Ui_Time
from win_list import Ui_List
from win_channels import Ui_Channels

#import portrait

win=None

class AboutDialog(MyMsgDialog):
    def __init__(self, *args, **kwargs):
	MyMsgDialog.__init__(self, *args, **kwargs)

	txt=self.tr("""
<html><style>
div.title {
    text-decoration:	underline;
}
</style>
<body>
<p> WifiEye is a program that scans for available wireless networks and
displays them in a nice way.

<div class="title">Copyright</div>
<p> Copyright &copy; 2010, Stefanos Harhalakis &lt;v13@v13.gr&gt;

<p> Send comments and bug reports to the above address.

<div class="title">Credit</div>
<p> Full credit for the idea goes to the creators of Wifi Analyzer, a
program for Android phones. WifiEye program is a clone of that one.

<div class="title">License</div>
<p> This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
<p> This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
<p> You should have received a copy of the GNU General Public License
along with this program.  If not, see
<a href="http://www.gnu.org/licenses/">http://www.gnu.org/licenses/</a>.
</body></html>
""")

	self.setWindowTitle(self.tr("About WifiEye"))

	self.ltitle=QLabel("WifiEye v" + config.version, self.w)
	self.ltitle.setObjectName("title")
	self.ltitle.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
	self.ltitle.setAlignment(Qt.AlignCenter)
	self.l.addWidget(self.ltitle)

	self.label=QLabel(txt, self.w)
	self.label.setWordWrap(True)
	self.label.setTextFormat(Qt.RichText)
	self.label.setAlignment(Qt.AlignJustify)
	self.l.addWidget(self.label)

	self.ltitle.setStyleSheet("""
	QLabel {
	    font-size:	    25pt;
	    color:	    rgb(192,192,192);
	    margin-bottom:  0.5ex;
	    }
	""")

class MainWin(QtGui.QMainWindow, maemo5.Portrait):
    def __init__(self, *args, **kwargs):
	QtGui.QMainWindow.__init__(self, *args, **kwargs)
	maemo5.Portrait.__init__(self)

	#self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
	#self.setAttribute(Qt.WA_Maemo5PortraitOrientation, True)
	#self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, True)

	self.ui={}
	self.what=None

	# An auto-refresh timer and its callback
	self.artimer=None
	self.artimer_cb=None

#	self.chtimer=None
#	self.tmtimer=None

	self.what=None

	self.dlgAbout=None

	self.setupUi()
	self.retranslateUi()
	self.selectUi("channels")
	#self.selectUi("list")

	#self.connect(QApplication.desktop(), SIGNAL('resized(int)'), self.slotResized)

#    def slotResized(self, n):
#	print "Resized", n

    def setupUiChannels(self):
	ui=Ui_Channels(self)
	ui.setupUi(self.centralwidget)

	self.connect(ui.buttonRefresh, SIGNAL('clicked()'),
		self.slotChRefresh)
	self.connect(ui.buttonAutoRefresh, SIGNAL('toggled(bool)'),
		self.slotChAutoRefresh)

	self.ui['channels']=ui

    def setupUiTime(self):
	ui=Ui_Time(self)
	ui.setupUi(self.centralwidget)

	self.connect(ui.buttonClear, SIGNAL('clicked()'),
		self.slotTmClear)
	self.connect(ui.buttonPause, SIGNAL('toggled(bool)'),
		self.slotTmPause)

	self.ui['time']=ui

    def setupUiList(self):
	ui=Ui_List(self)
	ui.setupUi(self.centralwidget)

	self.connect(ui.buttonRefresh, SIGNAL('clicked()'),
		self.slotLstRefresh)
	self.connect(ui.buttonAutoRefresh, SIGNAL('toggled(bool)'),
		self.slotLstAutoRefresh)

	self.ui['list']=ui

    def retranslateUi(self):
	self.setWindowTitle("WifiEye")
	self.ui['channels'].retranslateUi()
	self.ui['time'].retranslateUi()
	self.ui['list'].retranslateUi()

    def setupMenu(self):
	self.menuabout=QAction(self.tr('About'), self)
	self.menuabout.triggered.connect(self.menuAbout)

	m=self.menuBar()
	m.addAction(self.menuabout)


    def setupUi(self):
	sw=QStackedWidget(self)
	self.centralwidget=sw
	self.setCentralWidget(sw)

	self.setupMenu()
	self.setupUiChannels()
	self.setupUiTime()
	self.setupUiList()

	sw.addWidget(self.ui['channels'].w)
	sw.addWidget(self.ui['time'].w)
	sw.addWidget(self.ui['list'].w)

	# Do common signals
	for i in self.ui:
	    self.connect(self.ui[i].buttonWhat, SIGNAL('selected'),
		self.slotCommonWhat, Qt.QueuedConnection)

	maemo5.connectOrientationChange(self.slotCommonOrientationChanged)

    def selectUi(self, what):
	if self.what==what:
	    return

	self.what=what
	sw=self.centralwidget

	# Stop all timers first
	self.stopTimers()

	if what=='channels':
	    #self.stopTmTimer()
	    sw.setCurrentWidget(self.ui['channels'].w)
	    self.slotChRefresh()
	elif what=='time':
	    #self.stopChTimer()
	    sw.setCurrentWidget(self.ui['time'].w)
	    self.startTmTimer()
	elif what=='list':
	    sw.setCurrentWidget(self.ui['list'].w)
	    self.slotLstRefresh()
	else:
	    print "WTF? Bad what:", what

    # Internal function:
    # Start the auto-refresh timer and connect its timeout to the
    # specified callback function
    def startArTimer(self, cb):
	# If we don't need to change something then return
	if self.artimer!=None:
	    if self.artimer_cb==cb:
		return

	# Else stop it and recreate it
	self.stopArTimer()

	self.artimer=QtCore.QTimer(self)
	self.connect(self.artimer, SIGNAL('timeout()'), cb)
	self.artimer.start(1300)
	self.artimer_cb=cb

    def stopArTimer(self):
	if self.artimer==None:
	    return
	self.artimer.stop()
	self.artimer.deleteLater()
	self.artimer=None
	self.artimer_cb=None

    def startChTimer(self):
	self.startArTimer(self.slotChTimerAutorefresh)

#    def stopChTimer(self):
#	self.stopArTimer()

    def startTmTimer(self):
	self.startArTimer(self.slotTmTimerGetData)
	
    def stopTmTimer(self):
	self.stopArTimer()

    def startLstTimer(self):
	self.startArTimer(self.slotLstTimerAutorefresh)

#    def stopLstTimer(self):
#	self.stopArTimer()

    def stopTimers(self):
	self.stopArTimer()

    # -------------------------------
    # Slots:
    # slotCommon are Common slots
    # slotCh* are for Channel view
    # slotTm* are for Time view

    def slotCommonWhat(self):
	ui=self.ui[self.what]
	w=ui.buttonWhatSelection
	if w==self.what:
	    return

	self.selectUi(w)

    def slotCommonOrientationChanged(self):
	self.repaint()

    # -------------------------------

    # Common slot for Channel and List view
    # @param cb	    The callback for the timer
    def slotChLstRefresh(self, what, cb):
	ui=self.ui[what]
	if ui.buttonAutoRefresh.isChecked():
	    wifi.scanAsync(cb)
	else:
	    # Only disable the button when scanAsync() succeeds
	    # Else we may stay with a disabled button
	    if wifi.scanAsync(cb):
		ui.buttonRefresh.setEnabled(False)

    # Common slot for Channel and List view
    # @param what	channels/list
    # @pram startTimer	A function to call to start the timer
    def slotChLstAutoRefresh(self, what, startTimer, checked):
	ui=self.ui[what]
	if checked:
	    ui.buttonRefresh.setEnabled(False)
	    startTimer()
#	    if self.artimer==None:
#		self.startArTimer(cb)
	else:
	    ui.buttonRefresh.setEnabled(True)
	    self.stopArTimer()

    # Common slot for Channel and List view
    def slotChLstScanAsync(self, what, dt):
	ui=self.ui[what]
	if self.what!=what:
	    return
	if not ui.buttonAutoRefresh.isChecked():
	    ui.buttonRefresh.setEnabled(True)
	data.setone(dt)

    def slotChTimerAutorefresh(self):
	if self.what!='channels':
	    return
	self.slotChRefresh()

    # Button "refresh" pressed
    def slotChRefresh(self):
	self.slotChLstRefresh('channels', self.slotChScanAsync)

    # Button "auto-refresh" pressed
    def slotChAutoRefresh(self, checked):
	self.slotChLstAutoRefresh('channels', self.startChTimer, checked)

    # This is called asynchronously by the wifi scanner
    def slotChScanAsync(self, dt):
	self.slotChLstScanAsync('channels', dt)
	#self.repaint()
	ui=self.ui['channels']
	#ui.graph.repaint()
	ui.graph.update()

    # This is called when auto-refresh is enabled by the auto-refresh timer
    def slotChTimerAutorefresh(self):
	if self.what!='channels':
	    return
	self.slotChRefresh()

    # -------------------------------

    def slotLstRefresh(self):
	self.slotChLstRefresh('list', self.slotLstScanAsync)

    def slotLstAutoRefresh(self, checked):
	self.slotChLstAutoRefresh('list', self.startLstTimer, checked)

    def slotLstScanAsync(self, dt):
	self.slotChLstScanAsync('list', dt)
	ui=self.ui['list']
	ui.lst.update()

    def slotLstTimerAutorefresh(self):
	if self.what!='list':
	    return
	self.slotLstRefresh()

    # -------------------------------
    def slotTmTimerGetData(self):
	if self.what!='time':
	    return
	wifi.scanAsync(self.slotTmScanAsync)

    def slotTmClear(self):
	data.clear()
	#self.repaint()
	self.update()

    def slotTmScanAsync(self, dt):
	if self.what!='time':
	    return
	data.add(dt)
	#self.repaint()
	#self.ui['time'].graph.repaint()
	self.ui['time'].graph.update()

    def slotTmPause(self, checked):
	if checked:
	    self.stopTmTimer()
	else:
	    self.startTmTimer()

    def menuAbout(self):
	if self.dlgAbout==None:
	    self.dlgAbout=AboutDialog(self)

	ret=self.dlgAbout.exec_()

def init():
    global app, win, por
    app=QtGui.QApplication(sys.argv)

    win=MainWin()

def doit(dt):
    global app, win

    win.show()
    ret=app.exec_()
    wifi.finish()

    sys.exit(ret)

# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

