#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: win_common.py 2600 2010-06-02 18:53:23Z v13 $

__version__ = "$Id: win_common.py 2600 2010-06-02 18:53:23Z v13 $"

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import SIGNAL, Qt
from PyQt4.QtGui import *

import sys

whatOptions={
	    "channels":	    "Channels",
	    "time":	    "Time Graph",
	    "list":	    "Details",
    }
whatOptionsList=['channels', 'time', 'list']

# A Model for the available views
class WhatOptionsModel(QtCore.QAbstractListModel):
    def __init__(self, *args, **kwargs):
	QtCore.QAbstractListModel.__init__(self, *args, **kwargs)

    def data(self, index, role):
	global whatOptions, whatOptionsList

	if not index.isValid():
	    return(QtCore.QVariant())

	if index.row() >= len(whatOptions):
	    return(QtCore.QVariant())

	if role==Qt.DisplayRole:
	    id=whatOptionsList[index.row()]
	    return(whatOptions[id])

	return(QtCore.QVariant())

    def rowCount(self, parent):
	global whatOptionsList

	return(len(whatOptionsList))

    def getIdByIdx(self, idx):
	global whatOptionsList

	return(whatOptionsList[idx])

    def getIdxById(self, id):
	global whatOptionsList

	return(whatOptionsList.index(id))

# A dialog to select the desired view
class WhatPickerDialog(QDialog):
    def __init__(self, *args, **kwargs):
	QDialog.__init__(self, *args, **kwargs)

	#self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
	#self.setAttribute(Qt.WA_Maemo5PortraitOrientation, True)

	self.recreate()
	self.retranslateUi()

	self.selection=None	# The string representation of the current
				# selections ("channels", etc..)

	self.connect(self.lst, SIGNAL("clicked(const QModelIndex&)"),
	    self.slotListClicked, Qt.QueuedConnection)

    def recreate(self):
	self.l0=QHBoxLayout(self)

	self.lst=QListView(self)
	self.model=WhatOptionsModel(self.lst)
	self.lst.setModel(self.model)
	self.lst.setProperty("FingerScrollable", True)
	
	self.l0.addWidget(self.lst)

    def retranslateUi(self):
	self.setWindowTitle("Select view")

    def slotListClicked(self, index):
	idx=index.row()
	self.selection=self.model.getIdByIdx(idx)
	self.accept()

    # Set the current selected item.
    # sel is the string id ("channels", etc)
    def setSelection(self, sel):
	# Store this
	self.selection=sel
	# Get the index from the string
	idx=self.model.getIdxById(sel)
	# Convert it to QModelIndex
	idx2=self.model.index(idx, 0)
	# Set the selection
	self.lst.setCurrentIndex(idx2)

    def getSelection(self):
	return(self.selection)

# Common widgets between all views
class Ui_Common:
    def __init__(self, parent, what):
	self.what=what
	self.centralwidget=None
	self.slotButtonWhatDialog=None
	self.buttonWhatSelection=what
	self.w=None	    # The widget that should be shown
	self.parent=parent

    # @param what   The current view: "channels", etc
    def getCommonButtons(self):
	""" Return common buttons """
	global whatOptions, whatOptionsList

	ret=[]

	self.buttonWhat=QtGui.QPushButton()
	ret.append(self.buttonWhat)

	txt=whatOptions[self.what]
	#self.buttonWhat.setText(txt)
	self.buttonWhat.setText("Select view")

	QtCore.QObject.connect(self.buttonWhat, SIGNAL('clicked()'),
	    self.slotButtonWhat)

	return(ret)

    def slotButtonWhat(self):
	# Create the dialog (or reuse it)
	if self.slotButtonWhatDialog==None:
	    d=WhatPickerDialog(self.parent)
	    d.connect(d, SIGNAL('finished(int)'), self.slotButtonWhatFinished)
	else:
	    d=self.slotButtonWhatDialog

	# Set the correct current value
	d.setSelection(self.what)

	d.show()
	self.slotButtonWhatDialog=d

    def slotButtonWhatFinished(self, result):
	d=self.slotButtonWhatDialog

	# Get the selected item
	self.buttonWhatSelection=d.getSelection()

	#print "sel:", self.buttonWhatSelection

	# Emit a signal from the button
	self.buttonWhat.emit(QtCore.SIGNAL("selected"))

class MyMsgDialog(QDialog):
    """ Show a finger-scrollable message.
    Got from Maegirls """
    def __init__(self, *args, **kwargs):
	QDialog.__init__(self, *args, **kwargs)

	#self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)

	# This freaking thing is hard
	# It needs two layouts, one extra widget, the fingerscrollable
	# property set to true *and* setWidgetResizable(True)
	self._mm_l0=QVBoxLayout(self)

	self._mm_q=QScrollArea(self)
	self._mm_q.setWidgetResizable(True)
	self._mm_q.setProperty('FingerScrollable', True)

	self.w=QWidget(self._mm_q)

	self.l=QVBoxLayout(self.w)
	self._mm_q.setWidget(self.w)
	self._mm_l0.addWidget(self._mm_q)
	
# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

